package pwp;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import pwp.Match.Outcome;

public class PWPScraper {

    private List<Event> events;
    private Event currentEvent;
    private boolean inMatchHistory;

    public static void main(String[] args) throws IOException {
        PWPScraper scraper = new PWPScraper();

        Path path = Paths.get(args[0]);
        Charset charset = Charset.defaultCharset();

        try (BufferedReader reader = Files.newBufferedReader(path, charset)) {
            String line = null;

            while ((line = reader.readLine()) != null) {
                scraper.handleLine(line);
            }
        }

        scraper.report();
    }

    private PWPScraper() {
        events = new ArrayList<>();
        currentEvent = null;
        inMatchHistory = false;
    }

    private void handleLine(String line) {
        if (inMatchHistory) {
            handleMatchHistory(line);
        }
        else {
            handleEventInfo(line);
        }
    }

    private void handleMatchHistory(String line) {
        if (Character.isDigit(line.charAt(0))) {
            int outcomeIndex = Character.isDigit(line.charAt(1)) ? 2 : 1;
            int pointsIndex = line.indexOf("(");
            int opponentIndex = line.indexOf(")") + 2;

            Match match = new Match();
            match.setRound(Integer.parseInt(line.substring(0, outcomeIndex)));
            match.setOutcome(Outcome.fromString(line.substring(outcomeIndex, pointsIndex)));

            if (opponentIndex >= line.length()) {
                match.setOpponent("Multiple opponents");
            }
            else {
                match.setOpponent(line.substring(opponentIndex));
            }

            currentEvent.getMatches().add(match);
        }
        else if (line.startsWith("Multiple Opp")) {
            return;
        }
        else {
            inMatchHistory = false;
        }
    }

    private void handleEventInfo(String line) {
        if (line.startsWith("Event Type:")) {
            if (currentEvent != null) {
                events.add(currentEvent);
            }

            currentEvent = new Event();
            currentEvent.setType(line.split(":")[1].substring(1));
        }

        if (currentEvent == null) {
            return;
        }

        if (line.startsWith("Players:")) {
            currentEvent.setNumPlayers(Integer.parseInt(line.split(":")[1].substring(1)));
        }
        else if (line.startsWith("Format:")) {
            currentEvent.setFormat(line.split(":")[1].substring(1));
        }
        else if (line.startsWith("Place:")) {
            currentEvent.setPlace(Integer.parseInt(line.split(":")[1].substring(1)));
        }
        else if (line.startsWith("Match History:")) {
            inMatchHistory = true;
        }
    }

    private void report() {
        ReportByFormat report1 = new ReportByFormat(events);
        System.out.print(report1);

        System.out.println("-------------------------------------------------------\n");

        ReportByType report2 = new ReportByType(events);
        System.out.print(report2);

        System.out.println("-------------------------------------------------------\n");

        Record lifetimeRecord = Record.fromEvents(events);
        System.out.println("Lifetime record: " + lifetimeRecord);
        System.out.println("Win percentage ignoring draws: " + lifetimeRecord.winPercentageWithoutDraws());
        System.out.println("Win percentage including draws: " + lifetimeRecord.winPercentageWithDraws());
    }
}
